/*
 * PingResponseHandler.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.ping;

import com.veraxsystems.icmp.jni.IcmpEchoResponse;
import com.veraxsystems.icmp.ping.handlers.PingRequest;

/**
 * The {@link Ping} result handler interface. It is used for asynchronous handling
 * of Ping results. 
 */
public interface PingResponseHandler 
{
   /**
    * Method called by Ping on receiving proper ICMP echo response.
    * @param request the descriptor of request issued by {@link Ping#echo echo()} method
    * @param response ICMP response received
    */
   public void handleResponse(PingRequest request, IcmpEchoResponse response);

   /**
    * Method called by Ping in case timeout occurred when receiving ICMP echo response.
    * @param request the descriptor of request issued by {@link Ping#echo echo()} method
    */
   public void handleTimeout(PingRequest request);

   /**
    * Method called by Ping in case exception occurred when receiving ICMP echo response.
    * Such situation means fatal error of {@link Ping} class so it is very probable it
    * will not work any more.
    * @param request the descriptor of request issued by {@link Ping#echo echo()} method
    * @param exception exception occurred
    */
   public void handleException(PingRequest request, Exception exception);
   
}
