/*
 * ReceiveHandler.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.jni.handlers;

import com.veraxsystems.icmp.IcmpStatus;
import com.veraxsystems.icmp.jni.IcmpEchoResponse;
import com.veraxsystems.icmp.jni.IcmpJniBridge;
import com.veraxsystems.icmp.jni.exception.IcmpFatalErrorException;
import com.veraxsystems.icmp.jni.exception.IcmpSystemException;
import com.veraxsystems.icmp.jni.exception.IcmpTimeoutException;

/**
 * Internally used class for testing socket simultaneous send/receive
 * operation. This handler is responsible for receiving messages from provided 
 * IcmpJniBridge object.
 */
public class ReceiveHandler implements Runnable 
{
   IcmpJniBridge icmp;
   int requestsNumber;
   int responsesNumber;
   Counter handledCounter;

   /**
    * Initialize handler
    * @param icmpVal the ICMP object messages are received from
    * @param requestsNumberVal number or requested messages
    * @param handledCounterVal counter of sent request
    */
   public ReceiveHandler(IcmpJniBridge icmpVal, int requestsNumberVal, Counter handledCounterVal)
   {
      icmp = icmpVal;
      requestsNumber = requestsNumberVal;
      handledCounter = handledCounterVal;
   }

   public void run() 
   {
      int received = 0;
      responsesNumber = 0;
      System.out.println("ReceiveHandler.run IN ");
      try 
      {
         IcmpEchoResponse responseMsg;
         
         // Loop until all responses are received or timeout occurs
         while (responsesNumber < requestsNumber) 
         {
            icmp.selectSocket(10000);
            responseMsg = icmp.receive();
            responsesNumber += 1;
            handledCounter.inc();
            if (responseMsg.getStatus() != IcmpStatus.EchoReply) System.out.println("Type: " + responseMsg.getStatus());
            received += 1;
         }
         // Clean up
      }
      catch (IcmpTimeoutException ex) 
      {
         // Ignore it here - timeout is normal for ICMP
      }
      catch (IcmpFatalErrorException ex) 
      {
         ex.printStackTrace();
      }
      catch (IcmpSystemException ex) 
      {
         ex.printStackTrace();
      }
      finally 
      {
         System.out.println("Finally");
         System.out.println(" Received responses " + responsesNumber + " received: " + received);
      };
      System.out.println("ReceiveHandler.run OUT ");
   }
   
   
}
