/*
 * SendHandler.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.jni.handlers;

import com.veraxsystems.icmp.jni.IcmpJniBridge;
import com.veraxsystems.icmp.jni.exception.IcmpSystemException;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;

/**
 * Internally used class for testing socket simultaneous send/receive
 * operation. This handler is responsible for sending ICMP echo request 
 * via provided IcmpJniBridge object.
 */
public class SendHandler implements Runnable {
   private int requestsCount;
   Counter sentCounter;
   IcmpJniBridge icmp;


   /**
    * Initialize handler
    * @param icmpVal the ICMP object messages are received from
    * @param requestsNumberVal number of expected requests 
    * @param handledCounterVal counter of received responses
    */
   public SendHandler(IcmpJniBridge icmpVal, int requestsCountVal, Counter sendCounterVal) 
   {
      icmp = icmpVal;
      requestsCount = requestsCountVal;
      sentCounter = sendCounterVal;
   }

   
   public void run() 
   {
      byte[] dataToSend = new byte[128];
      int sent = 0;
      System.out.println("SendHandler.run IN ");
      try {
         // The pool of addresses used in test
         InetAddress[] addreses = new InetAddress[100];
         addreses[0] = InetAddress.getByName("192.168.9.1");
         addreses[1] = InetAddress.getByName("news.yahoo.com");
         addreses[2] = InetAddress.getByName("www.google.com");
         addreses[3] = InetAddress.getByName("8.8.8.8");
         addreses[4] = InetAddress.getByName("www.vertel.com");
         addreses[5] = InetAddress.getByName("www.cnn.com");
         addreses[6] = InetAddress.getByName("www.foxnews.com");
         addreses[7] = InetAddress.getByName("www.accuweather.com");
         addreses[8] = InetAddress.getByName("www.msnbc.com");
         addreses[9] = InetAddress.getByName("en.wikipedia.org");
         addreses[10] = InetAddress.getByName("www.hostitwise.com");
         addreses[11] = InetAddress.getByName("www.advancedgaragedoorinc.com");
         addreses[12] = InetAddress.getByName("www.bidobee.com");
         addreses[13] = InetAddress.getByName("www.imedicileather.com");
         addreses[14] = InetAddress.getByName("www.rushrash.com");
         addreses[15] = InetAddress.getByName("www.usatoday.com");
         addreses[16] = InetAddress.getByName("news.bbc.co.uk");
         addreses[17] = InetAddress.getByName("www.huffingtonpost.com");
         addreses[18] = InetAddress.getByName("www.ikea.com");
         addreses[19] = InetAddress.getByName("www.veraxsystems.com");

         // Some data to send
         Arrays.fill(dataToSend, (byte)123);
         while (sent < requestsCount) 
         {
            icmp.send((Inet4Address)addreses[sent % 20], icmp.getId(), dataToSend, dataToSend.length);
            // Add some timeout to not storm Ethernet with messages (just for performance test)
            try {
               Thread.sleep(1);
            } catch (InterruptedException ex) {
               // Nothing to handle
            }
            sent += 1;
            sentCounter.inc();
         };
         System.out.println("Echo sent " + sent);
         
         // Clean up
         System.out.println(" sent: " + sent);
      }
      catch (UnknownHostException ex) 
      {
         ex.printStackTrace();
      }
      catch (IcmpSystemException ex) 
      {
         ex.printStackTrace();
      }
      finally 
      {
         System.out.println("SendHandler.run OUT ");
      }
   }
   
   
}
